from ..base import BaseHTMLElement


class LabelElement(BaseHTMLElement):
    """
    LabelElement Class extends BaseHTMLElement to represent the HTML <label> element.

    HTML Use Cases:
    ---------------
    The HTML <label> element is used to associate a text label with a form control element.
    The association is done using the `for` attribute which matches the `id` of the form control.

    Methods:
    --------
    to_string():
        Converts the HTML element to an HTML string.

    Example Usage:
    --------------
    >>> label_elem = LabelElement(for_attribute="username", content="Username:")
    >>> print(label_elem.to_string())
    <label for="username">Username:</label>
    """

    def __init__(self, for_attribute: str = None, form: str = None, **kwargs) -> None:
        """
        Initializes a new LabelElement instance.

        Parameters:
        -----------
        for_attribute : str, optional
            Specifies the id of the form control with which the label is associated.
        form : str, optional
            Specifies the id of the form element the label is associated with.
        **kwargs : dict
            Additional keyword arguments that are passed to the parent class.

        Attributes:
        -----------
        for : str
            Represents the 'for' HTML attribute to associate the label with a form control.
        form : str
            Represents the 'form' HTML attribute to associate the label with a form.
        """
        attributes: dict[str, str] = {}
        if kwargs.get("attributes"):
            attributes.update(kwargs["attributes"])
            del kwargs["attributes"]
        attributes['for'] = for_attribute
        attributes['form'] = form
        super().__init__("label", attributes=attributes, **kwargs)


class LegendElement(BaseHTMLElement):
    """
    LegendElement Class extends BaseHTMLElement to represent the HTML <legend> element.

    HTML Use Cases:
    ---------------
    The HTML <legend> element is used to provide a title or caption within a <fieldset> element.
    It helps in grouping related controls in a form.

    Methods:
    --------
    to_string():
        Converts the HTML element to an HTML string.

    Example Usage:
    --------------
    >>> legend_elem = LegendElement(content="Personal Information")
    >>> print(legend_elem.to_string())
    <legend>Personal Information</legend>
    """

    def __init__(self, **kwargs) -> None:
        """
        Initializes a new LegendElement instance.

        Parameters:
        -----------
        **kwargs : dict
            Additional keyword arguments that are passed to the parent class.
        """
        super().__init__("legend", **kwargs)



class ListItemElement(BaseHTMLElement):
    """
    ListItemElement Class extends BaseHTMLElement to represent the HTML <li> element.

    HTML Use Cases:
    ---------------
    The HTML <li> element is used to represent list items within ordered (<ol>) or unordered (<ul>) lists.

    Methods:
    --------
    to_string():
        Converts the HTML element to an HTML string.

    Example Usage:
    --------------
    >>> li_elem = ListItemElement(content="Apple")
    >>> print(li_elem.to_string())
    <li>Apple</li>

    """

    def __init__(self, **kwargs) -> None:
        """
        Initializes a new ListItemElement instance.

        Parameters:
        -----------
        **kwargs : dict
            Additional keyword arguments that are passed to the parent class.
        """
        super().__init__("li", **kwargs)


class LinkElement(BaseHTMLElement):
    """
    LinkElement Class extends BaseHTMLElement to represent the HTML <link> element.

    HTML Use Cases:
    ---------------
    The HTML <link> element is used to define relationships between the current document and other resources.
    It is commonly used to link stylesheets, favicons, and other external resources.

    Methods:
    --------
    to_string():
        Converts the HTML element to an HTML string.

    Example Usage:
    --------------
    >>> link_elem = LinkElement(rel="stylesheet", href="/styles.css")
    >>> print(link_elem.to_string())
    <link rel="stylesheet" href="/styles.css">

    """

    def __init__(
            self,
            crossorigin: str = None,
            href: str = None,
            hreflang: str = None,
            media: str = None,
            referrerpolicy: str = None,
            rel: str = None,
            sizes: str = None,
            title: str = None,
            type: str = None,
            **kwargs
    ) -> None:
        """
        Initializes a new LinkElement instance.

        Parameters:
        -----------
        crossorigin : str, optional
            Specifies how the element handles crossorigin requests.
        href : str, optional
            Specifies the URL of the linked resource.
        hreflang : str, optional
            Language of the linked resource.
        media : str, optional
            Specifies what media/device the linked resource is optimized for.
        referrerpolicy : str, optional
            Referrer policy for the linked resource.
        rel : str, optional
            Relationship between the current document and the linked resource.
        sizes : str, optional
            Specifies the sizes of icons for visual media.
        title : str, optional
            Specifies extra information about the linked resource.
        type : str, optional
            Specifies the media type of the linked resource.
        **kwargs : dict
            Additional keyword arguments that are passed to the parent class.

        """
        attributes: dict[str, str] = {}
        if kwargs.get("attributes"):
            attributes.update(kwargs["attributes"])
            del kwargs["attributes"]
        attributes['crossorigin'] = crossorigin
        attributes['href'] = href
        attributes['hreflang'] = hreflang
        attributes['media'] = media
        attributes['referrerpolicy'] = referrerpolicy
        attributes['rel'] = rel
        attributes['sizes'] = sizes
        attributes['title'] = title
        attributes['type'] = type
        super().__init__("link", attributes=attributes, **kwargs)



from ..base import BaseHTMLElement


class FieldsetElement(BaseHTMLElement):
    """
    FieldsetElement Class extends BaseHTMLElement to represent HTML fieldset elements (`<fieldset>`).

    HTML Use Cases:
    ---------------
    The `<fieldset>` element is used to group together related elements in a form, typically via containing them within a `<legend>`.

    Methods:
    --------
    to_string():
        Converts the fieldset element to an HTML string.

    Example Usage:
    --------------
    >>> fieldset_elem = FieldsetElement(name="personal_info")
    >>> print(fieldset_elem.to_string())
    <fieldset name="personal_info"></fieldset>

    """

    def __init__(self, disabled: bool = False, form: str = None, name: str = None, **kwargs) -> None:
        """
        Initializes a new FieldsetElement instance.

        Parameters:
        -----------
        disabled : bool, optional
            Specifies whether the fieldset should be disabled.
        form : str, optional
            Specifies the form element that the fieldset belongs to.
        name : str, optional
            Specifies the name of the fieldset.
        **kwargs : dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class, such as 'content', 'attributes', or 'self_closing'.

        """
        attributes: dict[str, str | bool] = {}
        if kwargs.get("attributes"):
            attributes.update(kwargs["attributes"])
            del kwargs["attributes"]
        attributes["disabled"] = disabled
        attributes["form"] = form
        attributes["name"] = name
        super().__init__("fieldset", attributes=attributes, **kwargs)


class FigureCaptionElement(BaseHTMLElement):
    """
    FigureCaptionElement Class extends BaseHTMLElement to represent HTML figure caption elements (`<figcaption>`).

    HTML Use Cases:
    ---------------
    The `<figcaption>` element is used to provide a caption to an HTML `<figure>` element. It is usually
    placed as the first or the last child of the `<figure>`.

    Methods:
    --------
    to_string():
        Converts the figure caption element to an HTML string.

    Example Usage:
    --------------
    >>> figcaption_elem = FigureCaptionElement(content="This is a caption.")
    >>> print(figcaption_elem.to_string())
    <figcaption>This is a caption.</figcaption>

    """

    def __init__(self, **kwargs) -> None:
        """
        Initializes a new FigureCaptionElement instance.

        Parameters:
        -----------
        **kwargs : dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class, such as 'content', 'attributes', or 'self_closing'.
        """
        super().__init__("figcaption", **kwargs)


class FigureElement(BaseHTMLElement):
    """
    FigureElement Class extends BaseHTMLElement to represent HTML figure elements (`<figure>`).

    HTML Use Cases:
    ---------------
    The `<figure>` element is used to encapsulate media such as an image, diagram, or code snippet,
    along with its caption, provided by the `<figcaption>` element. It serves as a container for these
    media and allows them to be semantically grouped together.

    Methods:
    --------
    to_string():
        Converts the figure element to an HTML string.

    Example Usage:
    --------------
    >>> figure_elem = FigureElement(content=[image_elem, figcaption_elem])
    >>> print(figure_elem.to_string())
    <figure><img src="example.jpg" alt="An example image"><figcaption>This is a caption.</figcaption></figure>

    """

    def __init__(self, **kwargs) -> None:
        """
        Initializes a new FigureElement instance.

        Parameters:
        -----------
        **kwargs : dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class, such as 'content', 'attributes', or 'self_closing'.
        """
        super().__init__("figure", **kwargs)


class FooterElement(BaseHTMLElement):
    """
    FooterElement Class extends BaseHTMLElement to represent HTML footer elements (`<footer>`).

    HTML Use Cases:
    ---------------
    The `<footer>` element represents a container for the bottom section of a document or a section,
    typically containing information about the document such as author, copyright information, or links to related documents.

    Methods:
    --------
    to_string():
        Converts the footer element to an HTML string.

    Example Usage:
    --------------
    >>> footer_elem = FooterElement(content=["Copyright 2023"])
    >>> print(footer_elem.to_string())
    <footer>Copyright 2023</footer>

    """

    def __init__(self, **kwargs) -> None:
        """
        Initializes a new FooterElement instance.

        Parameters:
        -----------
        **kwargs : dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class, such as 'content', 'attributes', or 'self_closing'.
        """
        super().__init__("footer", **kwargs)


class FormElement(BaseHTMLElement):
    """
    FormElement Class extends BaseHTMLElement to represent HTML form elements (`<form>`).

    HTML Use Cases:
    ---------------
    The `<form>` element represents a section of a document containing interactive controls
    for submitting information to a web server. It can contain input elements like text fields,
    checkboxes, radio-buttons, submit buttons, etc.

    Methods:
    --------
    to_string():
        Converts the form element to an HTML string.

    Example Usage:
    --------------
    >>> form_elem = FormElement(action="/submit", method="post", content=["Submit your data"])
    >>> print(form_elem.to_string())
    <form action="/submit" method="post">Submit your data</form>

    """

    def __init__(
            self,
            accept_charset: str = None,
            action: str = None,
            autocomplete: str = None,
            enctype: str = None,
            method: str = None,
            name: str = None,
            novalidate: bool = False,
            rel: str = None,
            target: str = None,
            **kwargs
    ) -> None:
        """
        Initializes a new FormElement instance.

        Parameters:
        -----------
        accept_charset : str, optional
            Specifies the character encodings that are to be used for the form submission.
        action : str, optional
            Specifies where to send the form-data when a form is submitted.
        autocomplete : str, optional
            Specifies whether the <form> element should have autocomplete enabled.
        enctype : str, optional
            Specifies how the form-data should be encoded when submitting it to the server.
        method : str, optional
            Specifies the HTTP method to use when sending form-data.
        name : str, optional
            Specifies the name of the form.
        novalidate : bool, optional
            Specifies that the form should not be validated when submitted.
        rel : str, optional
            Specifies the relationship between the linked resource and the current document.
        target : str, optional
            Specifies where to display the response that is received after submitting the form.

        **kwargs : dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class, such as 'content', 'attributes', or 'self_closing'.
        """
        attributes: dict[str, str | bool] = {}
        if kwargs.get("attributes"):
            attributes.update(kwargs["attributes"])
            del kwargs["attributes"]
        attributes["accept-charset"] = accept_charset
        attributes["action"] = action
        attributes["autocomplete"] = autocomplete
        attributes["enctype"] = enctype
        attributes["method"] = method
        attributes["name"] = name
        attributes["novalidate"] = novalidate
        attributes["rel"] = rel
        attributes["target"] = target
        super().__init__("form", attributes=attributes, **kwargs)



from .tags import DoctypeElement


class HTML5Declaration(DoctypeElement):
    def __init__(self, **kwargs) -> None:
        super().__init__("html", **kwargs)


class HTML4_01StrictDeclaration(DoctypeElement):
    def __init__(self, **kwargs) -> None:
        super().__init__(
            'HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd"', **kwargs
        )


class HTML4_01TransitionalDeclaration(DoctypeElement):
    def __init__(self, **kwargs) -> None:
        super().__init__(
            'HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd"',
            **kwargs
        )


class XHTML1_0StrictDeclaration(DoctypeElement):
    def __init__(self, **kwargs) -> None:
        super().__init__(
            'html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" '
            '"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"',
            **kwargs
        )


class XHTML1_0TransitionalDeclaration(DoctypeElement):
    def __init__(self, **kwargs) -> None:
        super().__init__(
            'html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" '
            '"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"',
            **kwargs
        )


class XHTML1_1Declaration(DoctypeElement):
    def __init__(self, **kwargs) -> None:
        super().__init__(
            'html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"', **kwargs
        )


class HTML3_2Declaration(DoctypeElement):
    def __init__(self, **kwargs) -> None:
        super().__init__(
            'HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN"', **kwargs
        )



from .base import BaseHTMLElement
from .declarations import HTML5Declaration
from .tags import (TitleElement, HeadElement, BodyElement, HTMLElement)
from ..general_base import GeneralBaseElement


class HTML5Page(GeneralBaseElement):
    def __init__(self, title: str = "Untitled") -> None:
        self.head_elements: list[BaseHTMLElement] = [TitleElement(content=title)]
        self.body_elements: list[BaseHTMLElement] = []

    def add_to_head(self, element: BaseHTMLElement) -> None:
        self.head_elements.append(element)

    def add_to_body(self, element: BaseHTMLElement) -> None:
        self.body_elements.append(element)

    @property
    def _html_level_elements(self) -> list[BaseHTMLElement]:
        return [HeadElement(), BodyElement()]

    @property
    def _page_level_elements(self) -> list[BaseHTMLElement]:
        return [HTML5Declaration(), HTMLElement(content=self._html_level_elements)]

    def to_string(self) -> str:
        page_str: str = ""
        for element in self._page_level_elements:
            page_str += str(element)
        return page_str



from ..base import BaseHTMLElement


class GeneralHTMLElement(BaseHTMLElement):
    def __init__(
            self,
            tag_name: str,
            attributes: dict[str, any] = None,
            content: str | BaseHTMLElement | list[str | BaseHTMLElement] = None,
            self_closing: bool = False,
            declaration: bool = False
    ) -> None:
        super().__init__(
            tag_name=tag_name,
            attributes=attributes,
            content=content,
            self_closing=self_closing,
            declaration=declaration
        )


class CommentElement(BaseHTMLElement):
    """
    CommentElement Class extends BaseHTMLElement to represent HTML comments.
    The class takes a string as a comment and wraps it in an HTML comment tag `<!-- -->`.

    HTML Use Cases:
    ---------------
    HTML comments are useful to annotate your HTML source code, providing information
    about the code structure or explaining parts of the code for future reference.
    These comments are not displayed to the user on the web page.

    Attributes:
    -----------
    comment: str
        The text content of the HTML comment.

    Methods:
    --------
    to_string():
        Converts the comment to an HTML comment string.

    Example Usage:
    --------------
    >>> comment_elem = CommentElement("This is a sample comment")
    >>> print(comment_elem.to_string())
    <!--This is a sample comment-->

    """

    def __init__(self, comment: str, **kwargs) -> None:
        """
        Initializes a new CommentElement instance.

        Parameters:
        -----------
        comment: str
            The text content for the HTML comment.
        **kwargs: dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class.

        """
        super().__init__(f"!--{comment}--", self_closing=True, **kwargs)


class DoctypeElement(BaseHTMLElement):
    """
    DoctypeElement Class extends BaseHTMLElement to represent the DOCTYPE declaration.
    This class creates an HTML DOCTYPE element specifying the document type and version of HTML.

    HTML Use Cases:
    ---------------
    The DOCTYPE declaration is used to define the document type and version of HTML being used.
    It should be the very first thing in an HTML document, before the <html> element.

    Attributes:
    -----------
    declaration: str
        The HTML version or standard used, such as 'html' for <!DOCTYPE html>.
        Default is 'html'.

    Methods:
    --------
    to_string():
        Converts the DOCTYPE to a DOCTYPE string.

    Example Usage:
    --------------
    >>> doctype_elem = DoctypeElement("html")
    >>> print(doctype_elem.to_string())
    <!DOCTYPE html>

    >>> doctype_elem = DoctypeElement("HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"")
    >>> print(doctype_elem.to_string())
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">

    """

    def __init__(self, declaration: str = "html", **kwargs) -> None:
        """
        Initializes a new DoctypeElement instance.

        Parameters:
        -----------
        declaration: str
            The HTML version or standard being used. Default is 'html'.
        **kwargs: dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class.

        """
        super().__init__("!DOCTYPE", attributes={declaration: True}, declaration=True, **kwargs)



from ..base import BaseHTMLElement


class WordBreakOpportunityElement(BaseHTMLElement):
    """
    WordBreakOpportunityElement Class extends BaseHTMLElement to represent the HTML <wbr> element.

    HTML Use Cases:
    ---------------
    The HTML <wbr> element represents a word break opportunity, indicating where a word can be broken and wrapped to
    the next line when necessary.

    Methods:
    --------
    to_string():
        Converts the HTML element to an HTML string.

    Parameters:
    -----------
    **kwargs : dict
        Additional keyword arguments that are passed to the parent class.

    Example Usage:
    --------------
    # Create a WordBreakOpportunityElement
    >>> wbr = WordBreakOpportunityElement()

    # Convert it to an HTML string
    >>> print(wbr.to_string())
    <wbr>

    """

    def __init__(self, **kwargs) -> None:
        """
        Initializes a new WordBreakOpportunityElement instance.

        Parameters:
        -----------
        **kwargs : dict
            Additional keyword arguments that are passed to the parent class.

        """
        super().__init__("wbr", self_closing=True, **kwargs)



from ..base import BaseHTMLElement


class AnchorElement(BaseHTMLElement):
    """
    AnchorElement Class extends BaseHTMLElement to represent HTML anchor elements (`<a>`).
    This class creates an anchor element that can be used to define a hyperlink.

    HTML Use Cases:
    ---------------
    The anchor element is used to create hyperlinks to navigate to other web pages, files, or locations within the same page.
    It can also be used to activate JavaScript code.

    Attributes:
    -----------
    href: str
        Specifies the URL the link points to.
    download: str (optional)
        Specifies that the target will be downloaded when a user clicks on the hyperlink.
    hreflang: str (optional)
        Language of the linked resource.
    media: str (optional)
        Specifies what media/device the linked document is optimized for.
    ping: list[str] (optional)
        A space-separated list of URLs to which, when the link is followed, POST requests with the body PING will be sent by the browser.
    referrerpolicy: str (optional)
        Specifies which referrer information to send with the link.
    rel: str (optional)
        Specifies the relationship between the current document and the linked document.
    target: str (optional)
        Specifies where to open the linked document.
    type_attribute: str (optional)
        Specifies the media type of the linked document.

    Methods:
    --------
    to_string():
        Converts the anchor element to an HTML string.

    Example Usage:
    --------------
    >>> anchor_elem = AnchorElement(href="https://www.example.com")
    >>> print(anchor_elem.to_string())
    <a href="https://www.example.com"></a>

    >>> anchor_elem = AnchorElement(href="https://www.example.com", target="_blank", rel="noopener")
    >>> print(anchor_elem.to_string())
    <a href="https://www.example.com" target="_blank" rel="noopener"></a>

    """

    def __init__(
            self,
            href: str,
            download: str = None,
            hreflang: str = None,
            media: str = None,
            ping: list[str] = None,
            referrerpolicy: str = None,
            rel: str = None,
            target: str = None,
            type_attribute: str = None,
            **kwargs
    ) -> None:
        """
        Initializes a new AnchorElement instance.

        Parameters:
        -----------
        href: str
            The URL the link points to.
        download: str (optional)
            The download attribute.
        hreflang: str (optional)
            Language of the linked resource.
        media: str (optional)
            Media/device the linked document is optimized for.
        ping: list[str] (optional)
            List of URLs for ping attribute.
        referrerpolicy: str (optional)
            Referrer policy for the link.
        rel: str (optional)
            Relationship between current document and linked document.
        target: str (optional)
            Where to open the linked document.
        type_attribute: str (optional)
            Media type of the linked document.
        **kwargs: dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class.

        """
        attributes: dict[str, str] = {}
        if kwargs.get("attributes"):
            attributes.update(kwargs["attributes"])
            del kwargs["attributes"]

        attributes["href"] = href
        attributes["download"] = download
        attributes["hreflang"] = hreflang
        attributes["media"] = media
        attributes["ping"] = ' '.join(ping) if ping else None
        attributes["referrerpolicy"] = referrerpolicy
        attributes["rel"] = rel
        attributes["target"] = target
        attributes["type"] = type_attribute

        super().__init__("a", attributes=attributes, **kwargs)


class AbbreviationElement(BaseHTMLElement):
    """
    AbbreviationElement Class extends BaseHTMLElement to represent HTML abbreviation elements (`<abbr>`).
    This class creates an abbreviation or acronym element and allows specifying a title that describes the abbreviation.

    HTML Use Cases:
    ---------------
    The `<abbr>` HTML element is used to indicate abbreviations or acronyms. The title attribute can provide an
    expansion or description for the abbreviation, which can be displayed as a tooltip when the user hovers over it.

    Attributes:
    -----------
    title: str
        Specifies the full text that the abbreviation represents.
    abbreviation: str
        The abbreviation or acronym to be displayed.

    Methods:
    --------
    to_string():
        Converts the abbreviation element to an HTML string.

    Example Usage:
    --------------
    >>> abbr_elem = AbbreviationElement(title="HyperText Markup Language", abbreviation="HTML")
    >>> print(abbr_elem.to_string())
    <abbr title="HyperText Markup Language">HTML</abbr>

    """

    def __init__(self, title: str, abbreviation: str, **kwargs) -> None:
        """
        Initializes a new AbbreviationElement instance.

        Parameters:
        -----------
        title: str
            The full text that the abbreviation represents.
        abbreviation: str
            The abbreviation or acronym to be displayed.
        **kwargs: dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class.

        """
        attributes: dict[str, str] = {}
        if kwargs.get("attributes"):
            attributes.update(kwargs["attributes"])
            del kwargs["attributes"]

        attributes['title'] = title

        super().__init__("abbr", attributes=attributes, content=abbreviation, **kwargs)


class AddressElement(BaseHTMLElement):
    """
    AddressElement Class extends BaseHTMLElement to represent HTML address elements (`<address>`).
    This class creates an address element that can be used to define contact information.

    HTML Use Cases:
    ---------------
    The `<address>` HTML element is used to enclose contact information for a document or a section of a document.
    This element often appears in the footer of a document, a section, or an article.

    Methods:
    --------
    to_string():
        Converts the address element to an HTML string.

    Example Usage:
    --------------
    >>> address_elem = AddressElement(content=["1234 Elm Street", "Springfield, IL 62704"])
    >>> print(address_elem.to_string())
    <address>1234 Elm Street Springfield, IL 62704</address>

    >>> address_elem = AddressElement(content="Contact us at: <a href='mailto:webmaster@example.com'>webmaster@example.com</a>")
    >>> print(address_elem.to_string())
    <address>Contact us at: <a href='mailto:webmaster@example.com'>webmaster@example.com</a></address>

    """

    def __init__(self, **kwargs) -> None:
        """
        Initializes a new AddressElement instance.

        **kwargs: dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class, such as 'content'.

        """
        super().__init__("address", **kwargs)


class AreaElement(BaseHTMLElement):
    """
    AreaElement Class extends BaseHTMLElement to represent HTML area elements (`<area>`).
    This class creates an area element that defines a clickable area within an image map.

    HTML Use Cases:
    ---------------
    The `<area>` element is used within an `<map>` element to define clickable areas on an image.
    These areas can be linked to different URLs and activated when the user clicks on them.

    Attributes:
    -----------
    alt: str (optional)
        Alternate text describing the area. Useful for accessibility.
    coords: str (optional)
        Coordinates for the area's shape. Specifies the pixel coordinates for the shape attribute.
    download: str (optional)
        Specifies that the linked resource will be downloaded when clicked.
    href: str (optional)
        URL the clickable area leads to.
    hreflang: str (optional)
        Language of the linked resource.
    media: str (optional)
        Specifies the media type for which the linked resource is optimized.
    referrerpolicy: str (optional)
        Specifies which referrer information to send when following the link.
    rel: str (optional)
        Specifies the relationship between the current document and the linked resource.
    shape: str (optional)
        Specifies the shape of the clickable area (default, rect, circle, or poly).
    target: str (optional)
        Specifies where to open the linked document.
    type_attribute: str (optional)
        Specifies the media type of the linked resource.

    Methods:
    --------
    to_string():
        Converts the area element to an HTML string.

    Example Usage:
    --------------
    >>> area_elem = AreaElement(href="#", shape="rect", coords="34,44,270,350", alt="Computer")
    >>> print(area_elem.to_string())
    <area href="#" shape="rect" coords="34,44,270,350" alt="Computer" />

    """

    def __init__(
            self,
            alt: str = None,
            coords: str = None,
            download: str = None,
            href: str = None,
            hreflang: str = None,
            media: str = None,
            referrerpolicy: str = None,
            rel: str = None,
            shape: str = None,
            target: str = None,
            type_attribute: str = None,
            **kwargs
    ) -> None:
        """
        Initializes a new AreaElement instance.

        Parameters:
        -----------
        All optional attributes correspond to their HTML attribute counterparts.

        **kwargs: dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class.

        """
        attributes: dict[str, str] = {}
        if kwargs.get("attributes"):
            attributes.update(kwargs["attributes"])
            del kwargs["attributes"]

        attributes["alt"] = alt
        attributes["coords"] = coords
        attributes["download"] = download
        attributes["href"] = href
        attributes["hreflang"] = hreflang
        attributes["media"] = media
        attributes["referrerpolicy"] = referrerpolicy
        attributes["rel"] = rel
        attributes["shape"] = shape
        attributes["target"] = target
        attributes["type"] = type_attribute

        super().__init__("area", attributes=attributes, self_closing=True, **kwargs)


class ArticleElement(BaseHTMLElement):
    """
    ArticleElement Class extends BaseHTMLElement to represent HTML article elements (`<article>`).
    This class creates an article element that can be used to enclose a composition in a document,
    page, or application.

    HTML Use Cases:
    ---------------
    The `<article>` HTML element is used to enclose a self-contained composition like a blog post,
    a news story, or a forum post. An article element can contain other semantic elements like headers,
    footers, and sections, but should make sense on its own.

    Methods:
    --------
    to_string():
        Converts the article element to an HTML string.

    Example Usage:
    --------------
    >>> article_elem = ArticleElement(content=["<h1>Title</h1>", "<p>Content of the article.</p>"])
    >>> print(article_elem.to_string())
    <article><h1>Title</h1><p>Content of the article.</p></article>

    """

    def __init__(self, **kwargs) -> None:
        """
        Initializes a new ArticleElement instance.

        **kwargs: dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class, such as 'content'.

        """
        super().__init__("article", **kwargs)


class AsideElement(BaseHTMLElement):
    """
    AsideElement Class extends BaseHTMLElement to represent HTML aside elements (`<aside>`).
    This class creates an aside element that can be used to house content that is tangentially related
    to the content around it, and which can be considered separate from the main content.

    HTML Use Cases:
    ---------------
    The `<aside>` HTML element is used to contain content that is tangentially related to the content
    around it. While the content inside an `<aside>` could be relevant, removing it should not significantly
    affect the flow of the document. It's often used for sidebars, pull quotes, or advertising.

    Methods:
    --------
    to_string():
        Converts the aside element to an HTML string.

    Example Usage:
    --------------
    >>> aside_elem = AsideElement(content=["<h4>Related Links</h4>", "<ul><li><a href='#'>Link 1</a></li><li><a href='#'>Link 2</a></li></ul>"])
    >>> print(aside_elem.to_string())
    <aside><h4>Related Links</h4><ul><li><a href='#'>Link 1</a></li><li><a href='#'>Link 2</a></li></ul></aside>

    """

    def __init__(self, **kwargs) -> None:
        """
        Initializes a new AsideElement instance.

        **kwargs: dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class, such as 'content'.

        """
        super().__init__("aside", **kwargs)


class AudioElement(BaseHTMLElement):
    """
    AudioElement Class extends BaseHTMLElement to represent HTML audio elements (`<audio>`).
    This class creates an audio element that can embed audio content like songs, sound effects,
    or spoken text into HTML documents.

    HTML Use Cases:
    ---------------
    The `<audio>` HTML element is used to embed audio content in an HTML or XHTML document.
    The audio element provides options for autoplay, controlling the audio, looping, muting,
    and preloading, among others.

    Attributes:
    -----------
    autoplay: bool (optional)
        Specifies that the audio will start playing as soon as it is ready.
    controls: bool (optional)
        Specifies that audio controls should be displayed.
    loop: bool (optional)
        Specifies that the audio will start over again every time it is finished.
    muted: bool (optional)
        Specifies that the audio output should be muted.
    preload: bool (optional)
        Specifies if and how the author thinks the audio should be loaded.
    src: str (optional)
        Specifies the source URL of the audio file.

    Methods:
    --------
    to_string():
        Converts the audio element to an HTML string.

    Example Usage:
    --------------
    >>> audio_elem = AudioElement(controls=True, src="audio.mp3")
    >>> print(audio_elem.to_string())
    <audio controls src="audio.mp3"></audio>

    """

    def __init__(
            self,
            autoplay: bool = False,
            controls: bool = False,
            loop: bool = False,
            muted: bool = True,
            preload: bool = True,
            src: str = None,
            **kwargs
    ) -> None:
        """
        Initializes a new AudioElement instance.

        Parameters:
        -----------
        All optional attributes correspond to their HTML attribute counterparts.

        **kwargs: dict
            Optional keyword arguments inherited from the BaseHTMLElement parent class.

        """
        attributes: dict[str, str | bool] = {}
        if kwargs.get("attributes"):
            attributes.update(kwargs["attributes"])
            del kwargs["attributes"]

        attributes["autoplay"] = autoplay
        attributes["controls"] = controls
        attributes["loop"] = loop
        attributes["muted"] = muted
        attributes["preload"] = preload
        attributes["src"] = src

        super().__init__("audio", attributes=attributes, **kwargs)
